----------------------------------------
-- Search Patterns and Conf Variables --
----------------------------------------
local urlPattern;
do
	local ignoredKeys = " \"\'%(%)<>;\\%]%|";
	local nonBeginKeys = "[" .. ignoredKeys .. "]"; -- +ignoredKeys
	local userPrefix = "[%a%d][%a%d%.:_%-]*[%a%d]@";
	local domainLookaLike = "[%a%d][%a%d%.%-]*[%a%d]%.%a%a+"; -- + "%.%a+" ending + | [%a%d]+ begining
	local ipOctet = "[12]?%d?%d";
	local urlParams = "[/:?#][^" .. ignoredKeys .. "]+";
	local proto = "[%a%d]+://";
	local ip4 = (ipOctet .. "%."):rep(3) .. ipOctet;

	urlPattern = {
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. domainLookaLike  .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. ip4 .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. userPrefix .. domainLookaLike .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. userPrefix .. ip4 .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. domainLookaLike  .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. ip4 .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. userPrefix .. domainLookaLike .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. userPrefix .. ip4 .. urlParams .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. domainLookaLike .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. ip4 .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. userPrefix .. domainLookaLike .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. proto .. userPrefix .. ip4 .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. domainLookaLike .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. ip4 .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. userPrefix .. domainLookaLike .. ")%]*",
		"(" .. nonBeginKeys .. ")%[*(" .. userPrefix .. ip4 .. ")%]*",
	};
end
	
local ANNOUNCE_MSG_COPY = "NOW PRESS COPY-TO-CLIPBOARD HOTKEY TO COPY THE LINK";
local ANNOUNCE_MSG_INSERT = "LINK HAS BEEN INSERTED TO THE EDITBOX";
local ANNOUNCE_PAUSE_TIME = 5000;
local ANNOUNCE_FLASH_COUNTS = 7;
local ANNOUNCE_FLASH_SPEED = 45;
local ANNOUNCE_PRINT_SPEED = 10;

---------------------------
-- Parse and Click event --
---------------------------
local urlLink, announceLink;
local gsub = string.gsub;
local sub = string.sub;
local len = string.len;

urlLink = function(pre, str)
	local l = len(str);

	return pre .. "|H0:book:224:url|h[" .. 
		(l > 59 and sub(str, 1, 59) or str) .. "]|h" .. 
		(l > 59 and urlLink("[+]", sub(str, 59 + 1)) or "");
end

local initAddMessage = function()
	local forward = ZO_ChatSystem_GetEventHandlers()[EVENT_CHAT_MESSAGE_CHANNEL];
	local rawEventUpdate = ZO_ChatSystem_AddEventHandler;
	local urlPatternSize = #urlPattern;
	
	rawEventUpdate(EVENT_CHAT_MESSAGE_CHANNEL, function(arg1, arg2, msg, ...)
		if not msg then
			return forward(arg1, arg2, msg, ...);
		end
	
		msg = " " .. msg;

		for i = 1, urlPatternSize do
			msg = gsub(msg, urlPattern[i], urlLink);
		end
		
		return forward(arg1, arg2, sub(msg, 2), ...);
	end);
	
	ZO_ChatSystem_AddEventHandler = function(event, func)
		if event == EVENT_CHAT_MESSAGE_CHANNEL then
			forward = func;
		else
			rawEventUpdate(event, func);
		end
	end
end

local initLinkParser = function()
	local editbox = ZO_ChatWindowTextEntryEditBox;
	local forward = ZO_LinkHandler_OnLinkMouseUp;
	
	local copyLink = function(url, b)			
		if not b or b ~= 2 then
			editbox:TakeFocus();
			editbox:SetText(url);
			editbox:SelectAll();
			announceLink(ANNOUNCE_MSG_COPY);
		else
			editbox:InsertText(url);
			announceLink(ANNOUNCE_MSG_INSERT);
		end

		return true;
	end
	
	ZO_LinkHandler_OnLinkMouseUp = function(link, b, ...)
		if link:match("book:224:url") then
			return copyLink(link:match("%[(.-)%]"), b);
		else
			return forward(link, b, ...);
		end
	end
end

------------------
-- announceLink --
------------------
do
	local top = WINDOW_MANAGER:CreateTopLevelWindow"DerpURL";
	top:SetHeight(30);
    top:SetWidth(30);
    top:SetAnchor(BOTTOMLEFT, GuiRoot, CENTER, 26, -4);
	top:SetHidden(true);

	local label = WINDOW_MANAGER:CreateControl("DerpURL_MSG", top, CT_LABEL);
	label:SetFont"ZoFontHeader";
	label:SetHeight(40);
	label:SetAnchor(LEFT, top, LEFT, 0, 0);
	
	local limit, offset, buff, neg, tiks, amsg;
	
	local postUpdate = function()
		neg = not neg;
		top:SetHidden(neg);
		tiks = tiks + 1;
		
		if tiks > ANNOUNCE_FLASH_COUNTS then
			top:SetHidden(true);
			EVENT_MANAGER:UnregisterForUpdate"DerpURL";
		end
	end
	
	local pause = function()
		tiks, neg = 0, false;
		EVENT_MANAGER:UnregisterForUpdate"DerpURL";
		EVENT_MANAGER:RegisterForUpdate("DerpURL",  ANNOUNCE_FLASH_SPEED, postUpdate);
	end
	
	local update = function()
		buff = buff..sub(amsg, offset, offset)
		label:SetText(buff);
		offset = offset + 1;
		
		if offset > limit then
			EVENT_MANAGER:UnregisterForUpdate"DerpURL";
			EVENT_MANAGER:RegisterForUpdate("DerpURL", ANNOUNCE_PAUSE_TIME, pause);
		end
	end
	
	announceLink = function(msg)
		amsg = msg;
		limit = string.len(msg);
		offset, buff = 1, "";
		label:SetText"";
		top:SetHidden(false);
		EVENT_MANAGER:UnregisterForUpdate"DerpURL";
		EVENT_MANAGER:RegisterForUpdate("DerpURL", ANNOUNCE_PRINT_SPEED, update);
	end
end

------------------------
-- init DerpUrl addon --
------------------------
EVENT_MANAGER:RegisterForEvent("DerpURL", EVENT_PLAYER_ACTIVATED, function()
	EVENT_MANAGER:UnregisterForEvent("DerpURL", EVENT_PLAYER_ACTIVATED);
	EVENT_MANAGER:RegisterForUpdate("DerpURL", 1000, function()
		EVENT_MANAGER:UnregisterForUpdate"DerpURL";
		initLinkParser();
		initAddMessage();
	end);
end);
